#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-12-30
# File  : fritzbox_smarthome_device_lock.py (check plugin)
#
#

from typing import Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    Service,
    register,
    Result,
    State,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import CheckResult, DiscoveryResult
from cmk.base.plugins.agent_based.utils.fritzbox_smarthome import AvmSmartHomeDevice


def discovery_fritzbox_smarthome_device_lock_single(
        section: AvmSmartHomeDevice | Dict[str, AvmSmartHomeDevice]
) -> DiscoveryResult:
    if isinstance(section, AvmSmartHomeDevice):
        if section.lock is not None:
            yield Service()


def discovery_fritzbox_smarthome_device_lock_multiple(
        section: AvmSmartHomeDevice | Dict[str, AvmSmartHomeDevice]
) -> DiscoveryResult:
    if not isinstance(section, AvmSmartHomeDevice):
        for device_id, device in section.items():
            if device.lock is not None:
                yield Service(item=str(device_id))


def check_fritzbox_smarthome_device_lock_single(
        params, section: AvmSmartHomeDevice | Dict[str, AvmSmartHomeDevice]
) -> CheckResult:
    if not isinstance(section, AvmSmartHomeDevice):
        return

    if section.device_lock is None:
        return

    def _get_status(status: int):
        _dev_lock = {
            0: 'is not active',
            1: 'is active',
        }
        return _dev_lock.get(status, f'unknown ({status})')

    yield Result(state=State.OK, summary=f'Button lock on the device {_get_status(section.device_lock)}')


def check_fritzbox_smarthome_device_lock_multiple(
        item, params, section: AvmSmartHomeDevice | Dict[str, AvmSmartHomeDevice]
) -> CheckResult:
    if isinstance(section, Dict):
        try:
            yield from check_fritzbox_smarthome_device_lock_single(params, section[item])
        except KeyError:
            return


register.check_plugin(
    name='fritzbox_smarthome_device_lock_single',
    service_name='Device lock',
    sections=['fritzbox_smarthome'],
    discovery_function=discovery_fritzbox_smarthome_device_lock_single,
    check_function=check_fritzbox_smarthome_device_lock_single,
    # check_ruleset_name='fritzbox_smarthome_device_lock_single',
    check_default_parameters={}
)

register.check_plugin(
    name='fritzbox_smarthome_device_lock_multiple',
    service_name='Smarthome Device lock %s',
    sections=['fritzbox_smarthome'],
    discovery_function=discovery_fritzbox_smarthome_device_lock_multiple,
    check_function=check_fritzbox_smarthome_device_lock_multiple,
    # check_ruleset_name='fritzbox_smarthome_device_lock_multiple',
    check_default_parameters={}
)
